@@ -127,9 +127,15 @@ public class InitService extends SAMLServlet {
         }
 
         try {
+            // Create a new HttpSession if missing
+            httpRequest.getSession(true);
+
             String redirectURI;
             if (flow.equals("login") || flow.equals("relogin")) {
                 redirectURI = tryAutoLogin(httpRequest, httpResponse);
+                if (redirectURI == null) {
+                    redirectURI = provider.getStaticLoginRedirectLocation(httpRequest, httpResponse);
+                }
                 if (redirectURI == null) {
                     redirectURI = provider.buildAuthnRequest(httpRequest, httpResponse);
                 }
