@@ -108,6 +108,7 @@ import org.opensaml.xml.security.keyinfo.KeyInfoGenerator;
 import org.opensaml.xml.security.x509.X509KeyInfoGeneratorFactory;
 import org.opensaml.xml.signature.KeyInfo;
 import org.opensaml.xml.util.XMLHelper;
+import org.opensaml.xml.util.XMLObjectHelper;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.w3c.dom.Document;
@@ -118,6 +119,7 @@ import com.openexchange.dispatcher.DispatcherPrefixService;
 import com.openexchange.exception.OXException;
 import com.openexchange.groupware.notify.hostname.HostnameService;
 import com.openexchange.java.Charsets;
+import com.openexchange.java.Strings;
 import com.openexchange.java.util.UUIDs;
 import com.openexchange.saml.OpenSAML;
 import com.openexchange.saml.SAMLConfig;
@@ -125,6 +127,9 @@ import com.openexchange.saml.SAMLConfig.Binding;
 import com.openexchange.saml.SAMLExceptionCode;
 import com.openexchange.saml.SAMLSessionParameters;
 import com.openexchange.saml.SAMLWebSSOProvider;
+import com.openexchange.saml.oauth.service.OAuthAccessToken;
+import com.openexchange.saml.oauth.service.OAuthAccessTokenService;
+import com.openexchange.saml.oauth.service.OAuthAccessTokenService.OAuthGrantType;
 import com.openexchange.saml.spi.AuthenticationInfo;
 import com.openexchange.saml.spi.CredentialProvider;
 import com.openexchange.saml.spi.LogoutInfo;
@@ -187,12 +192,12 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
 
     private final ServiceLookup services;
 
-    public WebSSOProviderImpl(SAMLConfig config, OpenSAML openSAML, StateManagement stateManagement, ServiceLookup services) {
+    public WebSSOProviderImpl(SAMLConfig config, OpenSAML openSAML, StateManagement stateManagement, ServiceLookup services, SAMLBackend backend) {
         super();
         this.config = config;
         this.openSAML = openSAML;
         this.stateManagement = stateManagement;
-        this.backend = services.getService(SAMLBackend.class);
+        this.backend = backend;
         this.sessionReservationService = services.getService(SessionReservationService.class);
         this.services = services;
     }
@@ -255,6 +260,9 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
 
             enhanceAuthInfo(authInfo, bearerAssertion);
             Map<String, String> properties = authInfo.getProperties();
+
+            tryExchangeAssertionForOAuthToken(bearerAssertion, authInfo);
+
             String sessionToken = sessionReservationService.reserveSessionFor(
                 authInfo.getUserId(),
                 authInfo.getContextId(),
@@ -266,7 +274,7 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
                 .setScheme(getRedirectScheme(httpRequest))
                 .setHost(requestInfo.getDomainName())
                 .setPath(getRedirectPathPrefix() + "login")
-                .setParameter(LoginServlet.PARAMETER_ACTION, SAMLLoginTools.ACTION_SAML_LOGIN)
+                .setParameter(LoginServlet.PARAMETER_ACTION, SAMLLoginTools.ACTION_SAML_LOGIN + getPathString(backend.getPath()))
                 .setParameter(SAMLLoginTools.PARAM_TOKEN, sessionToken);
 
             String loginPath = requestInfo.getLoginPath();
@@ -345,6 +353,44 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
         return null;
     }
 
+    /**
+     *
+     * The given assertion is used to exchange it for an OAuth 2.0 token pair, by using the assertion as authorization grant
+     * as defined in RFC 7533: "Security Assertion Markup Language (SAML) 2.0 Profile for OAuth 2.0 Client Authentication and Authorization Grants"
+     *
+     * @param assertion The SAML {@link Assertion}
+     * @param authInfo The {@link AuthenticationInfo} of the user
+     * @throws OXException
+     */
+    private void tryExchangeAssertionForOAuthToken(Assertion assertion, AuthenticationInfo authInfo) throws OXException {
+        try {
+            OAuthAccessTokenService service = services.getService(OAuthAccessTokenService.class);
+            if(service==null){
+                LOG.debug("OAuthAccessTokenService is missing. Unable to exchange the assertion {} for an oauth token pair.", assertion.getID());
+                return;
+            }
+            if (!service.isConfigured(authInfo.getUserId(), authInfo.getContextId())) {
+                LOG.debug("OAuth token endpoint not properly configured. The assertion {} will not be exchanged for an oauth token pair.", assertion.getID());
+                return;
+            }
+
+            Assertion clonedAssertion = XMLObjectHelper.cloneXMLObject(assertion, true);
+            String xmlAssertion = openSAML.marshall(clonedAssertion);
+            String b64Assertion = Base64.encodeBase64URLSafeString(xmlAssertion.getBytes());
+            LOG.debug("Trying to exchange the assertion {} for an oauth token pair...", assertion.getID());
+            OAuthAccessToken token = service.getAccessToken(OAuthGrantType.SAML, b64Assertion, authInfo.getUserId(), authInfo.getContextId(), null);
+            authInfo.getProperties().put(SAMLSessionParameters.ACCESS_TOKEN, token.getAccessToken());
+            if(token.getRefreshToken()!=null){
+                authInfo.getProperties().put(SAMLSessionParameters.REFRESH_TOKEN, token.getRefreshToken());
+                LOG.debug("Successfully exchanged the assertion {} for an oauth access and refresh token.", assertion.getID());
+            } else {
+                LOG.debug("Successfully exchanged the assertion {} for an oauth access token.", assertion.getID());
+            }
+        } catch (MarshallingException | UnmarshallingException e) {
+            throw SAMLExceptionCode.INTERNAL_ERROR.create(e.getMessage());
+        }
+    }
+
     @Override
     public String buildLogoutRequest(HttpServletRequest httpRequest, HttpServletResponse httpResponse, Session session) throws OXException {
         final LogoutRequest logoutRequest = customizeLogoutRequest(prepareLogoutRequest(session), session, httpRequest, httpResponse);
@@ -389,7 +435,7 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
                 .setScheme(getRedirectScheme(httpRequest))
                 .setHost(requestInfo.getDomainName())
                 .setPath(getRedirectPathPrefix() + "login")
-                .setParameter(LoginServlet.PARAMETER_ACTION, SAMLLoginTools.ACTION_SAML_LOGOUT)
+                .setParameter(LoginServlet.PARAMETER_ACTION, SAMLLoginTools.ACTION_SAML_LOGOUT + getPathString(backend.getPath()))
                 .setParameter(LoginServlet.PARAMETER_SESSION, requestInfo.getSessionId())
                 .build();
 
@@ -414,7 +460,7 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
             validationStrategy.validateLogoutRequest(logoutRequest, httpRequest, binding);
             LogoutInfo logoutInfo = backend.resolveLogoutRequest(logoutRequest);
             LOG.debug("LogoutRequest is considered valid, starting to terminate sessions based on {}", logoutInfo);
-            terminateSessions(logoutRequest, logoutInfo, httpRequest, httpResponse);
+            terminateSessions(logoutRequest, logoutInfo);
             StatusCode statusCode = openSAML.buildSAMLObject(StatusCode.class);
             statusCode.setValue(StatusCode.SUCCESS_URI);
             status = openSAML.buildSAMLObject(Status.class);
@@ -482,9 +528,18 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
         spssoDescriptor.getAssertionConsumerServices().add(acs);
 
         if (config.singleLogoutEnabled()) {
+            String singleLogoutServiceURL = config.getSingleLogoutServiceURL();
+
+            // Set URI for SAML 2 POST binding
             SingleLogoutService slService = openSAML.buildSAMLObject(SingleLogoutService.class);
             slService.setBinding(SAMLConstants.SAML2_POST_BINDING_URI);
-            slService.setLocation(config.getSingleLogoutServiceURL());
+            slService.setLocation(singleLogoutServiceURL);
+            spssoDescriptor.getSingleLogoutServices().add(slService);
+
+            // Set URI for SAML 2 HTTP redirect binding
+            slService = openSAML.buildSAMLObject(SingleLogoutService.class);
+            slService.setBinding(SAMLConstants.SAML2_REDIRECT_BINDING_URI);
+            slService.setLocation(singleLogoutServiceURL);
             spssoDescriptor.getSingleLogoutServices().add(slService);
         }
 
@@ -681,7 +736,7 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
         }
     }
 
-    private void terminateSessions(LogoutRequest logoutRequest, LogoutInfo logoutInfo, HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws OXException {
+    private void terminateSessions(LogoutRequest logoutRequest, LogoutInfo logoutInfo) throws OXException {
         SessiondService sessiondService = services.getService(SessiondService.class);
         List<SessionIndex> sessionIndexes = logoutRequest.getSessionIndexes();
         boolean removedAnySession = false;
@@ -729,9 +784,21 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
         return secure ? "https" : "http";
     }
 
-    private void enhanceAuthInfo(AuthenticationInfo authInfo, Assertion bearerAssertion) {
+    private void enhanceAuthInfo(AuthenticationInfo authInfo, Assertion bearerAssertion) throws OXException {
+        Assertion clonedAssertion;
+        try {
+            // Using a cloned object because the opensaml.marshal method changes the assertion object
+            clonedAssertion = XMLObjectHelper.cloneXMLObject(bearerAssertion, true);
+        } catch (MarshallingException e) {
+            LOG.warn("Could not clone the assertion {}. Single logout for this session will probably fail.", bearerAssertion.getID(), e);
+            return;
+        } catch (UnmarshallingException e) {
+            LOG.warn("Could not clone the assertion {}. Single logout for this session will probably fail.", bearerAssertion.getID(), e);
+            return;
+        }
+
         Map<String, String> properties = authInfo.getProperties();
-        String sessionIndex = extractSessionIndex(bearerAssertion);
+        String sessionIndex = extractSessionIndex(clonedAssertion);
         if (sessionIndex != null) {
             /*
              * The SAML specification states that the bearer assertion must contain a <code>SessionIndex</code> attribute in its
@@ -746,7 +813,7 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
             properties.put(SAMLSessionParameters.SESSION_INDEX, sessionIndex);
         }
 
-        String sessionNotOnOrAfter = extractSessionNotOnOrAfter(bearerAssertion);
+        String sessionNotOnOrAfter = extractSessionNotOnOrAfter(clonedAssertion);
         if (sessionNotOnOrAfter != null) {
             /*
              * If an <AuthnStatement> used to establish a security context for the principal contains a SessionNotOnOrAfter attribute,
@@ -765,11 +832,33 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
          * element. We need to remember the elements XML representation, as it will be later on used to compile logout requests.
          * However if the IDP does not comply to the specification, the attribute might be missing.
          */
-        String subjectID = extractSubjectID(bearerAssertion);
+        String subjectID = extractSubjectID(clonedAssertion);
         if (subjectID != null) {
             properties.put(SAMLSessionParameters.SUBJECT_ID, subjectID);
         }
 
+        /*
+         * The bearer assertion might include an Attribute statement, with an attribute conforming to the xml format of the
+         * OAuth 2 specification.
+         */
+        String accessToken = extractAccessToken(clonedAssertion);
+        if (accessToken != null) {
+            properties.put(SAMLSessionParameters.ACCESS_TOKEN, accessToken);
+        }
+
+        /*
+         * The samlPath to be added to the session. Helps on resolving redirect and relogin situations in a multi-saml single host environment
+         */
+        String samlPath = backend.getPath();
+        if (!Strings.isEmpty(samlPath)) {
+            properties.put(SAMLSessionParameters.SAML_PATH, samlPath);
+        }
+
+        /*
+         * The singleLogut SAMLBackend parameter to be added to the session. Helps on resolving single logout situations in a multi-saml single host environment
+         */
+        properties.put(SAMLSessionParameters.SINGLE_LOGOUT, Boolean.toString(backend.getConfig().singleLogoutEnabled()));
+
         /*
          * This parameter must be checked within the AuthenticationService implementation. If not set, the login request is triggered
          * by any other authentication mechanism (e.g. HTTP Basic Auth) and its credentials must be checked.
@@ -1102,8 +1191,23 @@ public class WebSSOProviderImpl implements SAMLWebSSOProvider {
         return null;
     }
 
+    private String extractAccessToken(Assertion assertion) throws OXException {
+        return backend.getAccessToken(assertion);
+    }
+
     private static String generateID() {
         return UUIDs.getUnformattedString(UUID.randomUUID());
     }
 
+    private String getPathString(String path) {
+        if (Strings.isEmpty(path)) {
+            return "";
+        }
+        return path;
+    }
+
+    @Override
+    public String getStaticLoginRedirectLocation(HttpServletRequest httpRequest, HttpServletResponse httpResponse) {
+        return backend.getStaticLoginRedirectLocation(httpRequest, httpResponse);
+    }
 }
